# -*- coding: utf-8 -*-
from collections import OrderedDict
from .. import validators
from ..models import OneToOne, Result, classproperty, model_registry, mapper_registry, to_tuple
from ..utils import cached_property
from .gfk import GenericForeignKey

# Django-way with fields and local_fields???


class PolymorphicMapper(object):

    @cached_property
    def polymorphic_parent(self):
        for parent in self.model.mro():
            if parent is not self.model and parent in mapper_registry and getattr(mapper_registry[parent], 'polymorphic', False):
                return mapper_registry[parent]

    @classproperty
    def polymorphic_root(self):
        for parent in self.model.mro().reverse():
            if parent is not self.model and parent in mapper_registry and getattr(mapper_registry[parent], 'polymorphic', False):
                return mapper_registry[parent]

    @cached_property
    def polymorphic_columns(self):
        if self.polymorphic_parent:
            cols = self.polymorphic_parent.polymorphic_columns
        else:
            cols = OrderedDict()
        for k, v in self.columns.items():
            cols[k] = v
        return cols

    def _create_query(self):
        p = self.polymorphic_parent
        if p:
            t = self.sql_table
            q = p._create_query()
            q = q.fields(
                *self.get_sql_fields()
            ).tables((
                q.tables() & t
            ).on(
                t.pk == p.sql_table.pk
            ))
        else:
            q = super(PolymorphicMapper, self)._create_query()
        q.result = PolymorphicResult(self)
        return q

    def _do_prepare_model(self, model):
        for base in model.mro():
            if base is not model and getattr(mapper_registry.get(base), 'polymorphic', False):
                pk_rel_name = "{}_ptr".format(base.__name__.lower())
                # self.pk = "{}_id".format(pk_rel_name)  # Useless, pk read from DB
                setattr(model, pk_rel_name, OneToOne(
                    base,
                    field=mapper_registry[model].pk,
                    rel_field=mapper_registry[base].pk,
                    rel_name=model.__name__.lower(),
                    query=(lambda rel, owner: mapper_registry[rel.rel_model(owner)].query.polymorphic(False)),
                    rel_query=(lambda rel, owner: mapper_registry[rel.rel_model(owner)].query.polymorphic(False))
                ))
                break
        else:
            if getattr(mapper_registry[model], 'polymorphic', False):
                setattr(model, "real_instance", GenericForeignKey(
                    type_field="polymorphic_type_id",
                    field=mapper_registry[model].pk
                ))
        super(PolymorphicMapper, self)._do_prepare_model(self.model)

    def load(self, data, from_db=True):
        if from_db:
            cols = self.polymorphic_columns
            data_mapped = {}
            for key, value in data:
                try:
                    data_mapped[cols[key].name] = value
                except KeyError:
                    data_mapped[key] = value
        else:
            data_mapped = dict(data)
        obj = self.model(**data_mapped)
        self.set_original_data(obj, data_mapped)
        self.mark_new(obj, False)
        return obj

    def validate(self, obj, fields=frozenset(), exclude=frozenset()):
        errors = {}
        if self.polymorphic_parent:
            try:
                self.polymorphic_parent.validate(obj, fields=fields, exclude=exclude)
            except validators.ValidationError as e:
                errors.update(e.args[0])

        try:
            super(PolymorphicMapper, self).validate(obj, fields=fields, exclude=exclude)
        except validators.ValidationError as e:
            errors.update(e.args[0])

        if errors:
            raise validators.ValidationError(errors)

    def save(self, obj):
        if not getattr(obj, 'polymorphic_type_id', None):
            obj.polymorphic_type_id = mapper_registry[obj.__class__].name
        if self.polymorphic_parent:
            new_record = self.is_new(obj)
            self.polymorphic_parent.save(obj)
            for key, parent_key in zip(to_tuple(self.pk), to_tuple(self.polymorphic_parent.pk)):
                setattr(obj, key, getattr(obj, parent_key))
            self.mark_new(obj, new_record)
        return super(PolymorphicMapper, self).save(obj)


class PolymorphicResult(Result):

    _polymorphic = True

    def polymorphic(self, val=True):
        c = self._clone()
        c._polymorphic = val
        return c

    def fill_cache(self):
        if self._cache is not None or not self._polymorphic:
            return super(PolymorphicResult, self).fill_cache()

        if self._cache is None:
            polymorphic, self._polymorphic = self._polymorphic, False
            self._cache = list(self.iterator())
            populate_polymorphic(self._cache)
            self.populate_prefetch()
            self._polymorphic = polymorphic
        return self

    def iterator(self):
        for obj in super(PolymorphicResult, self).iterator():
            yield obj.real_instance if self._polymorphic and hasattr(obj, 'real_instance') else obj

    def _clone(self, *args, **kwargs):
        c = super(PolymorphicResult, self).clone(*args, **kwargs)
        c._polymorphic = self._polymorphic
        return c


def populate_polymorphic(rows):
    if not rows:
        return
    current_model = rows[0].__class__
    pks = {i.pk for i in rows}
    content_types = {i.polymorphic_type_id for i in rows}
    content_types -= set((mapper_registry[current_model].name,))
    typical_objects = {}
    for ct in content_types:
        model = model_registry[ct]
        typical_objects[ct] = {i.pk: i for i in mapper_registry[model].query.where(model.s.pk.in_(pks))}
    for i, obj in enumerate(rows):
        if obj.polymorphic_type_id in typical_objects:
            rows[i] = typical_objects[obj.polymorphic_type_id][obj.pk]



#!/usr/bin/env python
import unittest
from ascetic import validators
from ascetic import utils
from ascetic.databases import get_db
from ascetic.models import Model, ForeignKey, IdentityMap
from sqlbuilder import smartsql
from sqlbuilder.smartsql.tests import TestSmartSQL

TestSmartSQL

Author = Book = AuthorC = BookC = None


class TestValidators(unittest.TestCase):

    maxDiff = None

    def test_validators(self):
        ev = validators.Email()
        assert ev('test@example.com')
        assert not ev('adsf@.asdf.asdf')
        assert validators.Length()('a')
        assert not validators.Length(2)('a')
        assert validators.Length(max_length=10)('abcdegf')
        assert not validators.Length(max_length=3)('abcdegf')

        n = validators.Number(1, 5)
        assert n(2)
        assert not n(6)
        assert validators.Number(1)(100.0)
        assert not validators.Number()('rawr!')

        vc = validators.ValidatorChain(validators.Length(8), validators.Email())
        assert vc('test@example.com')
        assert not vc('a@a.com')
        assert not vc('asdfasdfasdfasdfasdf')


class TestUtils(unittest.TestCase):

    maxDiff = None

    def test_resolve(self):
        from ascetic.databases import Database
        self.assertTrue(utils.resolve('ascetic.databases.Database') is Database)


class TestModels(unittest.TestCase):

    maxDiff = None

    create_sql = {
        'postgresql': """
            DROP TABLE IF EXISTS ascetic_tests_author CASCADE;
            CREATE TABLE ascetic_tests_author (
                id serial NOT NULL PRIMARY KEY,
                first_name VARCHAR(40) NOT NULL,
                last_name VARCHAR(40) NOT NULL,
                bio TEXT
            );
            DROP TABLE IF EXISTS books CASCADE;
            CREATE TABLE books (
                id serial NOT NULL PRIMARY KEY,
                title VARCHAR(255),
                author_id integer REFERENCES ascetic_tests_author(id) ON DELETE CASCADE
            );
         """,
        'mysql': """
            DROP TABLE IF EXISTS ascetic_tests_author CASCADE;
            CREATE TABLE ascetic_tests_author (
                id INT(11) NOT NULL auto_increment,
                first_name VARCHAR(40) NOT NULL,
                last_name VARCHAR(40) NOT NULL,
                bio TEXT,
                PRIMARY KEY (id)
            );
            DROP TABLE IF EXISTS books CASCADE;
            CREATE TABLE books (
                id INT(11) NOT NULL auto_increment,
                title VARCHAR(255),
                author_id INT(11),
                FOREIGN KEY (author_id) REFERENCES ascetic_tests_author(id),
                PRIMARY KEY (id)
            );
         """,
        'sqlite3': """
            DROP TABLE IF EXISTS ascetic_tests_author;
            CREATE TABLE ascetic_tests_author (
              id INTEGER PRIMARY KEY AUTOINCREMENT,
              first_name VARCHAR(40) NOT NULL,
              last_name VARCHAR(40) NOT NULL,
              bio TEXT
            );
            DROP TABLE IF EXISTS books;
            CREATE TABLE books (
              id INTEGER PRIMARY KEY AUTOINCREMENT,
              title VARCHAR(255),
              author_id INT(11),
              FOREIGN KEY (author_id) REFERENCES ascetic_tests_author(id)
            );
        """
    }

    @classmethod
    def create_models(cls):
        class Author(Model):
            # books = OneToMany('ascetic.tests.models.Book')

            class Mapper(object):
                db_table = 'ascetic_tests_author'
                defaults = {'bio': 'No bio available'}
                validations = {'first_name': validators.Length(),
                               'last_name': (validators.Length(), lambda x: x != 'BadGuy!' or 'Bad last name', )}

        class Book(Model):
            author = ForeignKey(Author, rel_name='books')

            class Mapper(object):
                db_table = 'books'

        return locals()

    @classmethod
    def setUpClass(cls):
        db = get_db()
        db.cursor().execute(cls.create_sql[db.engine])
        for model_name, model in cls.create_models().items():
            globals()[model_name] = model

    def setUp(self):
        IdentityMap().disable()
        db = get_db()
        for table in ('ascetic_tests_author', 'books'):
            db.execute('DELETE FROM {0}'.format(db.qn(table)))

    def test_model(self):
        # Create tables

        # Test Creation
        james = Author(first_name='James', last_name='Joyce')
        james.save()

        kurt = Author(first_name='Kurt', last_name='Vonnegut')
        kurt.save()

        tom = Author(first_name='Tom', last_name='Robbins')
        tom.save()

        Book(title='Ulysses', author_id=james.id).save()
        Book(title='Slaughter-House Five', author_id=kurt.id).save()
        Book(title='Jitterbug Perfume', author_id=tom.id).save()
        slww = Book(title='Still Life with Woodpecker', author_id=tom.id)
        slww.save()

        # Test Model.pk getter and setter
        pk = slww.pk
        self.assertEqual(slww.pk, slww.id)
        slww.pk = tom.id
        self.assertEqual(slww.pk, tom.id)
        slww.pk = pk
        self.assertEqual(slww.pk, pk)

        self.assertTrue(kurt == Author.get(kurt.id))
        self.assertTrue(kurt != tom)

        # Test ForeignKey
        self.assertEqual(slww.author.first_name, 'Tom')
        slww.author = kurt
        self.assertEqual(slww.author.first_name, 'Kurt')
        del slww.author
        self.assertEqual(slww.author, None)
        slww.author = None
        self.assertEqual(slww.author, None)
        slww.author = tom.pk
        self.assertEqual(slww.author.first_name, 'Tom')

        # Test OneToMany
        self.assertEqual(len(list(tom.books)), 2)

        kid = kurt.id
        del(james, kurt, tom, slww)

        # Test retrieval
        b = Book.get(title='Ulysses')

        a = Author.get(id=b.author_id)
        self.assertEqual(a.id, b.author_id)

        a = Author.q.where(Author.s.id == b.id)[:]
        # self.assert_(isinstance(a, list))
        self.assert_(isinstance(a, smartsql.Q))

        # Test update
        new_last_name = 'Vonnegut, Jr.'
        a = Author.get(id=kid)
        a.last_name = new_last_name
        a.save()

        a = Author.get(kid)
        self.assertEqual(a.last_name, new_last_name)

        # Test count
        self.assertEqual(Author.q.count(), 3)
        self.assertEqual(len(Book.q.clone()), 4)
        self.assertEqual(len(Book.q.clone()[1:4]), 3)

        # Test delete
        a.delete()
        self.assertEqual(Author.q.count(), 2)
        self.assertEqual(len(Book.q.clone()), 3)

        # Test validation
        a = Author(first_name='', last_name='Ted')
        self.assertRaises(validators.ValidationError, a.validate)

        # Test defaults
        a.first_name = 'Bill and'
        a.save()
        self.assertEqual(a.bio, 'No bio available')

        a = Author(first_name='I am a', last_name='BadGuy!')
        self.assertRaises(validators.ValidationError, a.validate)

        print '### Testing for smartsql integration'
        t = Author.s
        fields = [t.q.result.db.compile(i)[0] for i in t.get_fields()]
        if get_db().engine == 'postgresql':
            self.assertListEqual(
                fields,
                ['"ascetic_tests_author"."id"',
                 '"ascetic_tests_author"."first_name"',
                 '"ascetic_tests_author"."last_name"',
                 '"ascetic_tests_author"."bio"', ]
            )
        else:
            self.assertListEqual(
                fields,
                ['`ascetic_tests_author`.`id`',
                 '`ascetic_tests_author`.`first_name`',
                 '`ascetic_tests_author`.`last_name`',
                 '`ascetic_tests_author`.`bio`', ]
            )

        if get_db().engine == 'postgresql':
            self.assertEqual(Book.s.author, '"book"."author_id"')
        else:
            self.assertEqual(Book.s.author, '`book`.`author_id`')

        q = t.q
        if get_db().engine == 'postgresql':
            self.assertEqual(q.result.db.compile(q)[0], '''SELECT "ascetic_tests_author"."id", "ascetic_tests_author"."first_name", "ascetic_tests_author"."last_name", "ascetic_tests_author"."bio" FROM "ascetic_tests_author"''')
        else:
            self.assertEqual(q.result.db.compile(q)[0], """SELECT `ascetic_tests_author`.`id`, `ascetic_tests_author`.`first_name`, `ascetic_tests_author`.`last_name`, `ascetic_tests_author`.`bio` FROM `ascetic_tests_author`""")
        self.assertEqual(len(q), 3)
        for obj in q:
            self.assertTrue(isinstance(obj, Author))

        q = q.where(t.id == b.author_id)
        if get_db().engine == 'postgresql':
            self.assertEqual(q.result.db.compile(q)[0], """SELECT "ascetic_tests_author"."id", "ascetic_tests_author"."first_name", "ascetic_tests_author"."last_name", "ascetic_tests_author"."bio" FROM "ascetic_tests_author" WHERE "ascetic_tests_author"."id" = %s""")
        else:
            self.assertEqual(q.result.db.compile(q)[0], """SELECT `ascetic_tests_author`.`id`, `ascetic_tests_author`.`first_name`, `ascetic_tests_author`.`last_name`, `ascetic_tests_author`.`bio` FROM `ascetic_tests_author` WHERE `ascetic_tests_author`.`id` = %s""")
        self.assertEqual(len(q), 1)
        self.assertTrue(isinstance(q[0], Author))

        # prefetch
        for obj in Book.q.prefetch('author').order_by(Book.s.id):
            self.assertTrue(hasattr(obj, '_cache'))
            self.assertTrue('author' in obj._cache)
            self.assertEqual(obj._cache['author'], obj.author)

        for obj in Author.q.prefetch('books').order_by(Author.s.id):
            self.assertTrue(hasattr(obj, '_cache'))
            self.assertTrue('books' in obj._cache)
            self.assertEqual(len(obj._cache['books']._cache), len(obj.books))
            for i in obj._cache['books']._cache:
                self.assertEqual(i._cache['author'], obj)


class TestCompositeRelation(unittest.TestCase):

    maxDiff = None

    create_sql = {
        'postgresql': """
            DROP TABLE IF EXISTS ascetic_composite_author CASCADE;
            CREATE TABLE ascetic_composite_author (
                id integer NOT NULL,
                lang VARCHAR(6) NOT NULL,
                first_name VARCHAR(40) NOT NULL,
                last_name VARCHAR(40) NOT NULL,
                bio TEXT,
                PRIMARY KEY (id, lang)
            );
            DROP TABLE IF EXISTS ascetic_composite_book CASCADE;
            CREATE TABLE ascetic_composite_book (
                id integer NOT NULL,
                lang VARCHAR(6) NOT NULL,
                title VARCHAR(255),
                author_id integer,
                PRIMARY KEY (id, lang),
                FOREIGN KEY (author_id, lang) REFERENCES ascetic_composite_author (id, lang) ON DELETE CASCADE
            );
         """,
        'mysql': """
            DROP TABLE IF EXISTS ascetic_composite_author CASCADE;
            CREATE TABLE ascetic_composite_author (
                id INT(11) NOT NULL,
                lang VARCHAR(6) NOT NULL,
                first_name VARCHAR(40) NOT NULL,
                last_name VARCHAR(40) NOT NULL,
                bio TEXT,
                PRIMARY KEY (id, lang)
            );
            DROP TABLE IF EXISTS ascetic_composite_book CASCADE;
            CREATE TABLE ascetic_composite_book (
                id INT(11) NOT NULL,
                lang VARCHAR(6) NOT NULL,
                title VARCHAR(255),
                author_id INT(11),
                PRIMARY KEY (id, lang),
                FOREIGN KEY (author_id, lang) REFERENCES ascetic_composite_author (id, lang)
            );
         """,
        'sqlite3': """
            DROP TABLE IF EXISTS ascetic_composite_author;
            CREATE TABLE ascetic_composite_author (
                id INTEGER NOT NULL,
                lang VARCHAR(6) NOT NULL,
                first_name VARCHAR(40) NOT NULL,
                last_name VARCHAR(40) NOT NULL,
                bio TEXT,
                PRIMARY KEY (id, lang)
            );
            DROP TABLE IF EXISTS ascetic_composite_book;
            CREATE TABLE ascetic_composite_book (
                id INTEGER NOT NULL,
                lang VARCHAR(6) NOT NULL,
                title VARCHAR(255),
                author_id INT(11),
                PRIMARY KEY (id, lang),
                FOREIGN KEY (author_id, lang) REFERENCES ascetic_composite_author (id, lang)
            );
        """
    }

    @classmethod
    def create_models(cls):
        class AuthorC(Model):
            # books = OneToMany('ascetic.tests.models.Book')

            class Mapper(object):
                db_table = 'ascetic_composite_author'
                defaults = {'bio': 'No bio available'}
                validations = {'first_name': validators.Length(),
                               'last_name': (validators.Length(), lambda x: x != 'BadGuy!' or 'Bad last name', )}

        class BookC(Model):
            author = ForeignKey(AuthorC, rel_field=('id', 'lang'), field=('author_id', 'lang'), rel_name='books')

            class Mapper(object):
                db_table = 'ascetic_composite_book'

        return locals()

    @classmethod
    def setUpClass(cls):
        db = get_db()
        db.cursor().execute(cls.create_sql[db.engine])
        for model_name, model in cls.create_models().items():
            globals()[model_name] = model

    def setUp(self):
        IdentityMap().disable()
        db = get_db()
        for table in ('ascetic_composite_author', 'ascetic_composite_book'):
            db.execute('DELETE FROM {0}'.format(db.qn(table)))

    def test_model(self):
        author = AuthorC(
            id=1,
            lang='en',
            first_name='First name',
            last_name='Last name',
        )
        author.save()
        author_pk = (1, 'en')
        author = AuthorC.get(author_pk)
        self.assertEqual(author.pk, author_pk)

        book = BookC(
            id=5,
            lang='en',
            title='Book title'
        )
        book.author = author
        book.save()
        book_pk = (5, 'en')
        book = BookC.get(book_pk)
        self.assertEqual(book.pk, book_pk)
        self.assertEqual(book.author.pk, author_pk)

        author = AuthorC.get(author_pk)
        self.assertEqual(author.books[0].pk, book_pk)



import collections
from ..models import Field, mapper_registry

# We can't use TranslationRegistry, because Mapper can be inherited, and we need to fix hierarchy???


class TranslationColumnDescriptor(object):

    def __get__(self, instance, owner):
        if not instance:
            return self
        return instance._mapper.translate_column(instance.original_column)

    def __set__(self, instance, value):
        instance.original_column = value.rsplit('_', 1)[0]


class TranslationField(Field):
    column = TranslationColumnDescriptor()


class TranslationMapper(object):

    translated_fields = ()

    def create_translation_field(self, name, data, declared_fields=None):
        field = super(TranslationMapper, self).create_field(name, data, declared_fields)
        if name in self.translated_fields and not isinstance(field, TranslationField):
            class NewField(TranslationField, field.__class__):
                pass
            field = NewField(**field.__dict__)
        return field

    def create_fields(self, columns, declared_fields):
        fields = collections.OrderedDict()
        rmap = {field.column: name for name, field in declared_fields.items() if hasattr(field, 'column')}

        original_columns = {}
        for name in self.translated_fields:
            if name in declared_fields and hasattr(declared_fields[name], 'column'):
                field = declared_fields[name]
                column = field.original_column if isinstance(field, TranslationField) else field.column
            else:
                column = name
            for lang in self.get_languages():
                original_columns[self.translate_column(column, lang)] = column

        for data in columns:
            column_name = data['column']
            column_name = original_columns.get(column_name, column_name)
            name = rmap.get(column_name, column_name)
            if name in fields:
                continue
            fields[name] = self.create_translation_field(name, data, declared_fields)

        for name, field in declared_fields.items():
            if name not in fields:
                fields[name] = self.create_translation_field(name, {'virtual': True}, declared_fields)
        return fields

    def add_field(self, name, field):
        field.name = name
        field._mapper = self
        self.fields[name] = field
        if isinstance(field, TranslationField):
            for lang in self.get_languages():
                self.columns[self.translate_column(field.original_column, lang)] = field
        else:
            self.columns[field.column] = field

    def _make_identity_key(self, model, pk):
        return super(TranslationMapper, self)._make_identity_key(model, pk) + (self.get_language(),)

    def translate_column(self, name, lang=None):
        return '{}_{}'.format(name, lang or self.get_language())

    def get_language(self):
        raise NotImplementedError

    def get_languages(self):
        raise NotImplementedError


"""
class TranslationRegistry(dict):

    _singleton = None

    def __new__(cls, *args, **kwargs):
        if not TranslationRegistry._singleton:
            if cls is TranslationRegistry:
                raise Exception("Can not create instance of abstract class {}".format(cls))
            TranslationRegistry._singleton = super(TranslationRegistry, cls).__new__(cls, *args, **kwargs)
        return TranslationRegistry._singleton

    def __call__(self, mapper, translated_fields):
        if mapper.name in self:
            raise Exception("Already registered {}".format(
                mapper.name)
            )
        self[mapper.name] = translated_fields
        mapper.fields = collections.OrderedDict()

        rmap = {field.column: name for name, field in mapper.declared_fields.items() if hasattr(field, 'column')}
        original_columns = {}
        for name in translated_fields:
            if name in mapper.declared_fields and hasattr(mapper.declared_fields[name], 'column'):
                column = mapper.declared_fields[name].column
            else:
                column = name
            for lang in self.get_languages():
                original_columns[self.translate_column(column, lang)] = column

        for column in mapper.columns:
            original_column = original_columns[column]
            name = rmap.get(original_column, original_column)
            if name in mapper.fields:
                field = mapper.fields[name]
            else:
                field = mapper.columns[column]
                if column in original_columns and not isinstance(field, TranslationField):
                    class NewTranslationField(TranslationField, field.__class__):
                        pass

                    data = vars(mapper.declared_fields[name]) if name in mapper.declared_fields else {}
                    data.update(vars(field))
                    field = NewTranslationField(**data)
            self.add_field(mapper, column, name, field)

        mapper.pk = mapper._read_pk(mapper.db_table, mapper._using, mapper.columns)
        mapper.sql_table = mapper._create_sql_table()
        mapper.base_query = mapper._create_base_query()
        mapper.query = mapper._create_query()

    def add_field(self, mapper, column, name, field):
        field.name = name
        field._mapper = mapper
        mapper.fields[name] = field
        mapper.columns[column] = field
"""



from __future__ import absolute_import
from sqlbuilder import smartsql
from ..models import ForeignKey, to_tuple, mapper_registry
from ..utils import cached_property

# Under construction!!! Not testet yet!!!


class MpMapper(object):
    """The simplest Materialized Path realization.

    Strong KISS principle.
    You shoul to create CHAR field with name "tree_path",
    and "parent_id".
    """

    PATH_SEPARATOR = '/'
    KEY_SEPARATOR = ':'
    PATH_DIGITS = 10

    def _do_prepare_model(self, model):
        setattr(model, 'parent', ForeignKey(
            'self',
            field=tuple('parent_{}'.format(k) for k in to_tuple(self.pk)),
            rel_name="children"
        ))

    def _mp_encode(self, value):
        return str(value).replace('&', '&a').replace(self.KEY_SEPARATOR, '&k').replace(self.PATH_SEPARATOR, '&p')

    def _mp_decode(self, value):
        return value.replace('&p', self.PATH_SEPARATOR).replace('&k', self.KEY_SEPARATOR).replace('&a', '&')

    @cached_property
    def mp_root(self):
        return mapper_registry[self.bound_relations['parent'].descriptor_class]

    def save(self, obj):
        """Sets content_type and calls parent method."""
        try:
            old_tree_path = self.get_original_data(obj)['tree_path']
        except (AttributeError, KeyError):
            old_tree_path = None

        super(MpMapper, self).save(obj)

        tree_path = self.KEY_SEPARATOR.join(self._mp_encode(i).zfill(self.PATH_DIGITS) for i in to_tuple(obj.pk))
        if obj.parent:
            tree_path = self.PATH_SEPARATOR.join((self.parent.tree_path, tree_path))

        if old_tree_path != tree_path:
            obj.tree_path = tree_path
            self.update_original_data(obj, tree_path=tree_path)
            self.mp_root.base_query.where(
                self.mp_root.sql_table.pk == obj.pk
            ).update({
                'tree_path': tree_path
            })

            if old_tree_path is not None:
                for obj in self.mp_root.base_query.where(
                    self.mp_root.sql_table.tree_path.startswith(old_tree_path) &
                    self.mp_root.sql_table.pk != obj.pk
                ).iterator():
                    self.mp_root.base_query.where(
                        self.mp_root.sql_table.pk == obj.pk
                    ).update({
                        'tree_path': tree_path + obj.tree_path[len(old_tree_path):]
                    })
        return self

    def get_ancestors_chained(self, obj, root=False, me=False, reverse=True):
        objs = []
        current = obj
        while (current if root else current.parent_id):
            if current != obj or me:
                objs.append(current)
            current = current.parent
        if reverse:
            objs.reverse()
        return objs

    def get_ancestors_by_path(self, obj, root=False, me=False, reverse=True):
        t = self.mp_root.sql_table
        q = self.query.where(
            smartsql.P(obj.tree_path).startswith(t.tree_path)
        )
        if not root:
            q = q.where(t.parent.is_not(None))
        if not me:
            q = q.where(t.pk != obj.pk)
        if reverse:
            q = q.order_by((t.tree_path,))
        else:
            q = q.order_by((t.tree_path.desc(),))
        return q

    def get_ancestors_by_paths(self, obj, root=False, me=False, reverse=True):
        q = self.query
        t = self.mp_root.sql_table
        cond = None
        paths = obj.tree_path.split(self.PATH_SEPARATOR)
        while paths:
            q = (t.tree_path == self.PATH_SEPARATOR.join(paths))
            cond = cond | q if cond is not None else q
            paths.pop()

        q = q.where(cond)

        if not root:
            q = q.where(t.parent.is_not(None))
        if not me:
            q = q.where(t.pk != obj.pk)

        if reverse:
            q = q.order_by(t.tree_path)
        else:
            q = q.order_by(t.tree_path.desc())
        return q

    def get_ancestors(self, obj, root=False, me=False, reverse=True):
        return self.get_ancestors_by_paths(obj, root=root, me=me, reverse=reverse)

    def get_hierarchical_name(self, obj, sep=', ', root=False, me=True, reverse=True, namegetter=unicode):
        """returns children QuerySet instance for given parent_id"""
        return sep.join(map(namegetter, self.get_ancestors(obj, root=root, me=me, reverse=reverse)))

    def get_children(self, obj):
        """Fix for MTI"""
        return self.query.where(self.mp_root.sql_table.parent == obj.pk)

    def _descendants(self, obj):
        r = list(self.get_children(obj))
        for i in r:
            r += i._descendants(obj)
        return r

    def get_descendants_recursive(self, obj, me=False):
        r = []
        if me:
            r.append(obj)
        r += self._descendants(obj)
        return r

    def get_descendants(self, obj, me=False):
        t = self.mp_root.sql_table
        q = self.query.where(t.tree_path.startswith(obj.tree_path))
        if not me:
            q = q.where(t.pk != obj.pk)
        return q


class MpModel(object):

    def get_ancestors(self, root=False, me=False, reverse=True):
        return mapper_registry[self.__class__].get_ancestors(self, root, me, reverse)

    def get_hierarchical_name(self, sep=', ', root=False, me=True, reverse=True, namegetter=unicode):
        return mapper_registry[self.__class__].get_hierarchical_name(self, sep, root, me, reverse, namegetter)

    def get_children(self):
        return mapper_registry[self.__class__].get_children(self)

    def get_descendants(self, me=False):
        return mapper_registry[self.__class__].get_descendants(self, me)



import unittest
from ascetic.databases import get_db
from ascetic.models import Model, IdentityMap, mapper_registry
from ascetic.contrib.modeltranslation import TranslationMapper

Author = None


class TranslationMapper(TranslationMapper):

    _language = 'ru'

    def get_languages(self):
        return ('ru', 'en')

    def get_language(self):
        return self._language


class TestModelTranslation(unittest.TestCase):

    maxDiff = None

    create_sql = {
        'postgresql': """
            DROP TABLE IF EXISTS ascetic_modeltranslation_author CASCADE;
            CREATE TABLE ascetic_modeltranslation_author (
                id serial NOT NULL PRIMARY KEY,
                first_name_en VARCHAR(40),
                first_name_ru VARCHAR(40),
                last_name_en VARCHAR(40),
                last_name_ru VARCHAR(40),
                bio TEXT
            );
         """,
        'mysql': """
            DROP TABLE IF EXISTS ascetic_modeltranslation_author CASCADE;
            CREATE TABLE ascetic_modeltranslation_author (
                id INT(11) NOT NULL auto_increment,
                first_name_en VARCHAR(40),
                first_name_ru VARCHAR(40),
                last_name_en VARCHAR(40),
                last_name_ru VARCHAR(40),
                bio TEXT,
                PRIMARY KEY (id)
            );
         """,
        'sqlite3': """
            DROP TABLE IF EXISTS ascetic_modeltranslation_author;
            CREATE TABLE ascetic_modeltranslation_author (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                first_name VARCHAR(40),
                first_name VARCHAR(40),
                last_name VARCHAR(40),
                last_name VARCHAR(40),
              bio TEXT
            );
        """
    }

    @classmethod
    def create_models(cls):

        class Author(Model):
            class Mapper(TranslationMapper):
                db_table = 'ascetic_modeltranslation_author'
                map = {'first_alias': 'first_name'}
                defaults = {'bio': 'No bio available'}
                translated_fields = ('first_alias', 'last_name')

        return locals()

    @classmethod
    def setUpClass(cls):
        db = get_db()
        db.cursor().execute(cls.create_sql[db.engine])
        for model_name, model in cls.create_models().items():
            globals()[model_name] = model

    def setUp(self):
        IdentityMap().disable()
        db = get_db()
        for table in ('ascetic_modeltranslation_author',):
            db.execute('DELETE FROM {0}'.format(db.qn(table)))

    def test_meta(self):
        mapper = mapper_registry[Author]
        self.assertIn('id', mapper.fields)
        self.assertIn('bio', mapper.fields)
        self.assertIn('first_alias', mapper.fields)
        self.assertIn('last_name', mapper.fields)
        for lang in mapper.get_languages():
            self.assertNotIn('first_alias_{}'.format(lang), mapper.fields)
            self.assertNotIn('last_name_{}'.format(lang), mapper.fields)

        self.assertIn('id', mapper.columns)
        self.assertIn('bio', mapper.columns)
        for lang in mapper.get_languages():
            self.assertIn('first_name_{}'.format(lang), mapper.columns)
            self.assertIn('last_name_{}'.format(lang), mapper.columns)

        for lang in mapper.get_languages():
            self.assertNotIn('first_name', mapper.columns)
            self.assertNotIn('last_name', mapper.columns)

        self.assertEqual(len(mapper.fields), 4)
        self.assertEqual(len(mapper.columns), 6)

        current_language = mapper.get_language()
        for lang in mapper.get_languages():
            mapper._language = lang
            self.assertEqual(mapper.fields['id'].column, 'id')
            self.assertEqual(mapper.fields['bio'].column, 'bio')
            self.assertEqual(mapper.fields['first_alias'].column, 'first_name_{}'.format(lang))
            self.assertEqual(mapper.fields['last_name'].column, 'last_name_{}'.format(lang))
        mapper._language = current_language

    def test_model(self):
        author = Author(first_alias='First name', last_name='Last name')
        author.save()
        author = Author.get(author.id)
        self.assertEqual(author.first_alias, 'First name')
        self.assertEqual(author.last_name, 'Last name')



import os

DATABASES_MYSQL = {
    'default': {
        'engine': "mysql",
        'user': "devel",
        'db': "devel_ascetic",
        'passwd': "devel",
        'debug': True,
        'initial_sql': "SET NAMES 'UTF8';",
    }
}

DATABASES_POSTGRESQL = {
    'default': {
        'engine': "postgresql",
        'user': "devel",
        'database': "devel_ascetic",
        'password': "devel",
        'debug': True,
        'initial_sql': "SET NAMES 'UTF8';",
    }
}

DATABASES = DATABASES_POSTGRESQL

DEBUG = True

LOGGER_INIT = 'ascetic.settings.init_logger'


def init_logger(settings):
    import logging
    if settings.DEBUG:
        import warnings
        warnings.simplefilter('default')

        logger = logging.getLogger('ascetic')
        logger.setLevel(logging.DEBUG)
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        logger.addHandler(ch)

try:
    m = __import__(os.getenv('ASCETIC_SETTINGS', 'ascetic_settings'))
except ImportError:
    pass
else:
    for key in dir(m):
        if key[0] != '_':
            globals()[key] = getattr(m, key)


def configure(settings):
    globals().update(settings)



import collections
from ..models import ForeignKey, OneToMany, cascade, model_registry, mapper_registry, to_tuple, is_model_instance


class GenericForeignKey(ForeignKey):

    def __init__(self, type_field="object_type_id", rel_field=None, field=None, on_delete=cascade, rel_name=None, query=None):
        self._type_field = type_field
        self._rel_field = rel_field and to_tuple(rel_field)
        self._field = field and to_tuple(field)
        self.on_delete = on_delete
        self._rel_name = rel_name
        self._query = query

    def type_field(self, owner):
        return self._type_field

    def field(self, owner):
        return self._field or ('object_id',)

    def rel_field(self, owner):
        return self._rel_field

    def rel_name(self, owner):
        if self._rel_name is None:
            return '{0}_set'.format(self.model(owner).__name__.lower())
        elif isinstance(self._rel_name, collections.Callable):
            return self._rel_name(self, owner)
        else:
            return self._rel_name

    def rel_model(self, instance):
        if isinstance(instance, type):
            raise TypeError('"instance" argument should be instance of model, not class.')
        return model_registry[getattr(instance, self.type_field(instance.__class__))]

    def query(self, instance):
        if isinstance(instance, type):
            raise TypeError('"instance" argument should be instance of model, not class.')
        if isinstance(self._query, collections.Callable):
            return self._query(self, instance)
        else:
            return mapper_registry[self.rel_model(instance)].query

    @property
    def add_related(self):
        raise AttributeError

    def __get__(self, instance, owner):
        if not instance:
            return self
        val = tuple(getattr(instance, f) for f in self.field(owner))

        if not [i for i in val if i is not None]:
            return None

        cached_obj = self._get_cache(instance, self.name(owner))
        rel_field = self.rel_field(owner)
        rel_model = self.rel_model(instance)
        if not isinstance(cached_obj, rel_model) or tuple(getattr(cached_obj, f, None) for f in rel_field) != val:
            if self._query is None and rel_field == to_tuple(mapper_registry[rel_model].pk):
                obj = rel_model.get(val)  # to use IdentityMap
            else:
                t = mapper_registry[rel_model].sql_table
                q = self.query(instance)
                for f, v in zip(rel_field, val):
                    q = q.where(t.__getattr__(f) == v)
                obj = q[0]
            self._set_cache(instance, self.name(owner), obj)
        return instance._cache[self.name(owner)]

    def __set__(self, instance, value):
        owner = instance.__class__
        if is_model_instance(value):
            setattr(instance, self.type_field(owner), mapper_registry[value.__class__].name)
            self._set_cache(instance, self.name(owner), value)
            value = tuple(getattr(value, f) for f in self.rel_field(owner))
        value = to_tuple(value)
        for a, v in zip(self.field(owner), value):
            setattr(instance, a, v)

    def __delete__(self, instance):
        owner = instance.__class__
        self._set_cache(instance, self.name(owner), None)
        setattr(instance, self.type_field(instance.__class__), None)
        for a in self.field(owner):
            setattr(instance, a, None)


class GenericRelation(OneToMany):

    def field(self, owner):
        rel_model = self.rel_model(owner)
        return getattr(rel_model, self.rel_name(owner)).rel_field(rel_model)

    def rel_field(self, owner):
        rel_model = self.rel_model(owner)
        return getattr(rel_model, self.rel_name(owner)).field(rel_model)

    def rel_type_field(self, owner):
        rel_model = self.rel_model(owner)
        return getattr(rel_model, self.rel_name(owner)).type_field(rel_model)

    def __get__(self, instance, owner):
        if not instance:
            return self
        rel_field = self.rel_field(owner)
        rel_type_field = self.rel_type_field(owner)
        val = tuple(getattr(instance, f) for f in self.field(owner))
        cached_query = self._get_cache(instance, self.name(owner))
        # TODO: Be sure that value of related fields equals to value of field
        if cached_query is not None and cached_query._cache is not None:
            for cached_obj in cached_query._cache:
                if (tuple(getattr(cached_obj, f, None) for f in rel_field) != val or
                        getattr(cached_obj, rel_type_field) != mapper_registry[type(instance)].name):
                    cached_query = None
                    break
        if cached_query is None:
            t = mapper_registry[self.rel_model(owner)].sql_table
            q = super(GenericRelation, self).__get__(instance, owner)
            q = q.where(t.__getattr__(rel_type_field) == mapper_registry[instance.__class__].name)
            self._set_cache(instance, self.name(owner), q)
        return self._get_cache(instance, self.name(owner))

    def __set__(self, instance, object_list):
        owner = instance.__class__
        rel_field = self.rel_field(owner)
        rel_type_field = self.rel_type_field(owner)
        val = tuple(getattr(instance, f) for f in self.field(owner))
        for cached_obj in object_list:
            if is_model_instance(cached_obj):
                if not isinstance(cached_obj, self.rel_model(owner)):
                    raise Exception(
                        'Value should be an instance of "{0}" or primary key of related instance.'.format(
                            mapper_registry[self.rel_model(owner)].name
                        )
                    )
                if (tuple(getattr(cached_obj, f, None) for f in rel_field) != val or
                        getattr(cached_obj, rel_type_field) != mapper_registry[instance.__class__].name):
                    return
        self.__get__(instance, owner)._cache = object_list



import unittest
from ascetic import validators
from ascetic.databases import get_db
from ascetic.contrib.polymorphic import PolymorphicMapper
from ascetic.models import Model, ForeignKey, IdentityMap, mapper_registry

Author = Book = Nonfiction = Avia = None


class TestModelTranslation(unittest.TestCase):

    maxDiff = None

    create_sql = {
        'postgresql': """
            DROP TABLE IF EXISTS ascetic_polymorphic_author CASCADE;
            CREATE TABLE ascetic_polymorphic_author (
                id integer NOT NULL,
                lang VARCHAR(6) NOT NULL,
                first_name VARCHAR(40) NOT NULL,
                last_name VARCHAR(40) NOT NULL,
                bio TEXT,
                PRIMARY KEY (id, lang)
            );
            DROP TABLE IF EXISTS ascetic_polymorphic_book CASCADE;
            CREATE TABLE ascetic_polymorphic_book (
                id integer NOT NULL,
                lang VARCHAR(6) NOT NULL,
                title VARCHAR(255),
                author_id integer,
                polymorphic_type_id VARCHAR(255),
                PRIMARY KEY (id, lang),
                FOREIGN KEY (author_id, lang) REFERENCES ascetic_polymorphic_author (id, lang) ON DELETE CASCADE
            );
            DROP TABLE IF EXISTS ascetic_polymorphic_nonfiction CASCADE;
            CREATE TABLE ascetic_polymorphic_nonfiction (
                nonfiction_ptr_id integer NOT NULL,
                nonfiction_ptr_lang VARCHAR(6) NOT NULL,
                branch VARCHAR(255),
                PRIMARY KEY (nonfiction_ptr_id, nonfiction_ptr_lang),
                FOREIGN KEY (nonfiction_ptr_id, nonfiction_ptr_lang) REFERENCES ascetic_polymorphic_book (id, lang) ON DELETE CASCADE
            );
            DROP TABLE IF EXISTS ascetic_polymorphic_avia CASCADE;
            CREATE TABLE ascetic_polymorphic_avia (
                avia_ptr_id integer NOT NULL,
                avia_ptr_lang VARCHAR(6) NOT NULL,
                model VARCHAR(255),
                PRIMARY KEY (avia_ptr_id, avia_ptr_lang),
                FOREIGN KEY (avia_ptr_id, avia_ptr_lang) REFERENCES ascetic_polymorphic_nonfiction (nonfiction_ptr_id, nonfiction_ptr_lang) ON DELETE CASCADE
            );
         """,
        'mysql': """
            DROP TABLE IF EXISTS ascetic_polymorphic_author CASCADE;
            CREATE TABLE ascetic_polymorphic_author (
                id INT(11) NOT NULL,
                lang VARCHAR(6) NOT NULL,
                first_name VARCHAR(40) NOT NULL,
                last_name VARCHAR(40) NOT NULL,
                bio TEXT,
                PRIMARY KEY (id, lang)
            );
            DROP TABLE IF EXISTS ascetic_polymorphic_book CASCADE;
            CREATE TABLE ascetic_polymorphic_book (
                id INT(11) NOT NULL,
                lang VARCHAR(6) NOT NULL,
                title VARCHAR(255),
                author_id INT(11),
                polymorphic_type_id VARCHAR(255),
                PRIMARY KEY (id, lang),
                FOREIGN KEY (author_id, lang) REFERENCES ascetic_polymorphic_author (id, lang)
            );
            DROP TABLE IF EXISTS ascetic_polymorphic_nonfiction CASCADE;
            CREATE TABLE ascetic_polymorphic_nonfiction (
                nonfiction_ptr_id INT(11) NOT NULL,
                nonfiction_ptr_lang VARCHAR(6) NOT NULL,
                branch VARCHAR(255),
                PRIMARY KEY (nonfiction_ptr_id, nonfiction_ptr_lang),
                FOREIGN KEY (nonfiction_ptr_id, nonfiction_ptr_lang) REFERENCES ascetic_polymorphic_book (id, lang) ON DELETE CASCADE
            );
            DROP TABLE IF EXISTS ascetic_polymorphic_avia CASCADE;
            CREATE TABLE ascetic_polymorphic_avia (
                avia_ptr_id INT(11) NOT NULL,
                avia_ptr_lang VARCHAR(6) NOT NULL,
                model VARCHAR(255),
                PRIMARY KEY (avia_ptr_id, avia_ptr_lang),
                FOREIGN KEY (avia_ptr_id, avia_ptr_lang) REFERENCES ascetic_polymorphic_nonfiction (nonfiction_ptr_id, nonfiction_ptr_lang) ON DELETE CASCADE
            );
         """,
        'sqlite3': """
            DROP TABLE IF EXISTS ascetic_polymorphic_author;
            CREATE TABLE ascetic_polymorphic_author (
                id INTEGER NOT NULL,
                lang VARCHAR(6) NOT NULL,
                first_name VARCHAR(40) NOT NULL,
                last_name VARCHAR(40) NOT NULL,
                bio TEXT,
                PRIMARY KEY (id, lang)
            );
            DROP TABLE IF EXISTS ascetic_polymorphic_book;
            CREATE TABLE ascetic_polymorphic_book (
                id INTEGER NOT NULL,
                lang VARCHAR(6) NOT NULL,
                title VARCHAR(255),
                author_id INT(11),
                polymorphic_type_id VARCHAR(255),
                PRIMARY KEY (id, lang),
                FOREIGN KEY (author_id, lang) REFERENCES ascetic_polymorphic_author (id, lang)
            );
            DROP TABLE IF EXISTS ascetic_polymorphic_nonfiction CASCADE;
            CREATE TABLE ascetic_polymorphic_nonfiction (
                nonfiction_ptr_id INTEGER NOT NULL,
                nonfiction_ptr_lang VARCHAR(6) NOT NULL,
                branch VARCHAR(255),
                PRIMARY KEY (nonfiction_ptr_id, nonfiction_ptr_lang),
                FOREIGN KEY (nonfiction_ptr_id, nonfiction_ptr_lang) REFERENCES ascetic_polymorphic_book (id, lang) ON DELETE CASCADE
            );
            DROP TABLE IF EXISTS ascetic_polymorphic_avia CASCADE;
            CREATE TABLE ascetic_polymorphic_avia (
                avia_ptr_id INTEGER NOT NULL,
                avia_ptr_lang VARCHAR(6) NOT NULL,
                model VARCHAR(255),
                PRIMARY KEY (avia_ptr_id, avia_ptr_lang),
                FOREIGN KEY (avia_ptr_id, avia_ptr_lang) REFERENCES ascetic_polymorphic_nonfiction (nonfiction_ptr_id, nonfiction_ptr_lang) ON DELETE CASCADE
            );
        """
    }

    @classmethod
    def create_models(cls):

        class Author(Model):

            class Mapper(object):
                db_table = 'ascetic_polymorphic_author'
                defaults = {'bio': 'No bio available'}
                validations = {'first_name': validators.Length(),
                               'last_name': (validators.Length(), lambda x: x != 'BadGuy!' or 'Bad last name', )}

        class Book(Model):
            author = ForeignKey(
                Author,
                rel_field=('id', 'lang'),
                field=('author_id', 'lang'),
                rel_name='books',
                rel_query=(lambda rel, owner: mapper_registry[rel.rel_model(owner)].query)
            )

            class Mapper(PolymorphicMapper):
                name = 'ascetic.contrib.tests.test_polymorphic.Book'
                db_table = 'ascetic_polymorphic_book'
                polymorphic = True

        class Nonfiction(Book):

            class Mapper(PolymorphicMapper):
                name = 'ascetic.contrib.tests.test_polymorphic.Nonfiction'
                db_table = 'ascetic_polymorphic_nonfiction'
                polymorphic = True

        class Avia(Nonfiction):

            class Mapper(PolymorphicMapper):
                name = 'ascetic.contrib.tests.test_polymorphic.Avia'
                db_table = 'ascetic_polymorphic_avia'
                polymorphic = True

        return locals()

    @classmethod
    def setUpClass(cls):
        db = get_db()
        db.cursor().execute(cls.create_sql[db.engine])
        for model_name, model in cls.create_models().items():
            globals()[model_name] = model

    def setUp(self):
        IdentityMap().disable()
        db = get_db()
        for table in ('ascetic_polymorphic_author', 'ascetic_polymorphic_book',):
            db.execute('DELETE FROM {0}'.format(db.qn(table)))

    def test_book(self):
        author = Author(
            id=1,
            lang='en',
            first_name='First name',
            last_name='Last name',
        )
        author.save()
        author_pk = (1, 'en')
        author = Author.get(author_pk)
        self.assertEqual(author.pk, author_pk)

        book = Book(
            id=5,
            lang='en',
            title='Book title'
        )
        book.author = author
        book.save()
        book_pk = (5, 'en')
        book = Book.get(book_pk)
        self.assertEqual(book.pk, book_pk)
        self.assertEqual(book.author.pk, author_pk)

        author = Author.get(author_pk)
        self.assertEqual(author.books[0].pk, book_pk)

    def test_nonfiction(self):
        author = Author(
            id=1,
            lang='en',
            first_name='First name',
            last_name='Last name',
        )
        author.save()
        author_pk = (1, 'en')
        author = Author.get(author_pk)
        self.assertEqual(author.pk, author_pk)

        nonfiction_data = dict(
            id=5,
            lang='en',
            title='Book title',
            branch='instruction',
        )
        nonfiction = Nonfiction(**nonfiction_data)
        nonfiction.author = author
        nonfiction.save()
        nonfiction_pk = (5, 'en')

        nonfiction = Book.get(nonfiction_pk)
        self.assertIsInstance(nonfiction, Nonfiction)
        for k, v in nonfiction_data.items():
            self.assertEqual(getattr(nonfiction, k), v)
        self.assertEqual(nonfiction.pk, nonfiction_pk)
        self.assertEqual(nonfiction.author.pk, author_pk)
        self.assertEqual(author.books[0], nonfiction)

        nonfiction = Nonfiction.get(nonfiction_pk)
        self.assertIsInstance(nonfiction, Nonfiction)
        for k, v in nonfiction_data.items():
            self.assertEqual(getattr(nonfiction, k), v)
        self.assertEqual(nonfiction.pk, nonfiction_pk)
        self.assertEqual(nonfiction.author.pk, author_pk)
        self.assertEqual(author.books[0], nonfiction)

    def test_avia(self):
        author = Author(
            id=1,
            lang='en',
            first_name='First name',
            last_name='Last name',
        )
        author.save()
        author_pk = (1, 'en')
        author = Author.get(author_pk)
        self.assertEqual(author.pk, author_pk)

        avia_data = dict(
            id=5,
            lang='en',
            title='Book title',
            branch='instruction',
            model='An'
        )
        avia = Avia(**avia_data)
        avia.author = author
        avia.save()
        avia_pk = (5, 'en')

        avia = Book.get(avia_pk)
        self.assertIsInstance(avia, Nonfiction)
        for k, v in avia_data.items():
            self.assertEqual(getattr(avia, k), v)
        self.assertEqual(avia.pk, avia_pk)
        self.assertEqual(avia.author.pk, author_pk)
        self.assertEqual(author.books[0], avia)

        avia = Avia.get(avia_pk)
        self.assertIsInstance(avia, Nonfiction)
        for k, v in avia_data.items():
            self.assertEqual(getattr(avia, k), v)
        self.assertEqual(avia.pk, avia_pk)
        self.assertEqual(avia.author.pk, author_pk)
        self.assertEqual(author.books[0], avia)



from __future__ import absolute_import
import collections
import logging
from time import time
from functools import wraps
from threading import local
from uuid import uuid4
from sqlbuilder import smartsql
from sqlbuilder.smartsql.compilers import mysql, sqlite
from ascetic import settings
from ascetic.utils import resolve

try:
    str = unicode  # Python 2.* compatible
    string_types = (basestring,)
    integer_types = (int, long)
except NameError:
    string_types = (str,)
    integer_types = (int,)

PLACEHOLDER = '%s'

resolve(settings.LOGGER_INIT)(settings)


class Transaction(object):

    def __init__(self, using='default'):
        """Constructor of Transaction instance."""
        self.db = databases[using]

    def __call__(self, f=None):
        if f is None:
            return self

        @wraps(f)
        def _decorated(*args, **kw):
            with self:
                rv = f(*args, **kw)
            return rv

        return _decorated

    def __enter__(self):
        self.db.begin()

    def __exit__(self, exc_type, exc_val, exc_tb):
        try:
            if exc_type:
                self.db.rollback()
            else:
                try:
                    self.db.commit()
                except:
                    self.db.rollback()
                    raise
        finally:
            pass


class Database(object):

    _engines = {}
    placeholder = '%s'
    compile = smartsql.compile
    connection = None
    _begin_level = 0
    _autocommit = False

    def __init__(self, alias, engine, initial_sql, always_reconnect=False, autocommit=False, debug=False, **kwargs):
        self.alias = alias
        self.engine = engine
        self.debug = debug
        self.initial_sql = initial_sql
        self.always_reconnect = always_reconnect
        self.autocommit = autocommit
        self._savepoints = []
        self._conf = kwargs
        self._logger = logging.getLogger('.'.join((__name__, self.alias)))

        if self.debug:
            self._execute = self.log_sql(self._execute)

    def connection_factory(self, **kwargs):
        raise NotImplementedError

    def _ensure_connected(self):
        self.connection = self.connection_factory(**self._conf)
        if self.autocommit:
            self.set_autocommit(self.autocommit)
        if self.initial_sql:
            self.connection.cursor().execute(self.initial_sql)
        return self

    def log_sql(self, f):
        alias = self.alias
        logger = self._logger

        @wraps(f)
        def wrapper(sql, params=()):
            start = time()
            try:
                return f(sql, params)
            except Exception as e:
                logger.exception(e)
                raise
            finally:
                stop = time()
                duration = stop - start
                logger.debug(
                    '%s - (%.4f) %s; args=%s' % (alias, duration, sql, params),
                    extra={'alias': alias, 'duration': duration, 'sql': sql, 'params': params}
                )
        return wrapper

    def _execute(self, sql, params=()):
        cursor = self.cursor()
        try:
            cursor.execute(sql, params)
        except Exception:
            if (not self._autocommit or self._begin_level > 0) and not self.always_reconnect:
                raise
            self._ensure_connected()
            cursor = self.cursor()
            cursor.execute(sql, params)
        return cursor

    def execute(self, sql, params=()):
        if not isinstance(sql, string_types):
            sql, params = self.compile(sql)
        if self.placeholder != PLACEHOLDER:
            sql = sql.replace(PLACEHOLDER, self.placeholder)
        sql = sql.rstrip("; \t\n\r")
        cursor = self._execute(sql, params)
        return cursor

    def cursor(self):
        if not self.connection:
            self._ensure_connected()
        return self.connection.cursor()

    def last_insert_id(self, cursor):
        return cursor.lastrowid

    def get_autocommit(self):
        return self._autocommit and self._begin_level == 0

    def set_autocommit(self, autocommit=True):
        self._autocommit = autocommit
        return self

    @property
    def transaction(self):
        return Transaction(self.alias)

    def begin(self):
        if self._begin_level == 0:
            self.execute("BEGIN")
        else:
            self.savepoint_begin()
        self._begin_level += 1

    def commit(self):
        self._begin_level = max(0, self._begin_level - 1)
        if self._begin_level == 0:
            self.connection.commit()
            from ascetic.models import IdentityMap
            IdentityMap().clear()
        else:
            self.savepoint_commit()

    def rollback(self):
        self._begin_level = max(0, self._begin_level - 1)
        if self._begin_level == 0:
            self.connection.rollback()
            from ascetic.models import IdentityMap
            IdentityMap().clear()
        else:
            self.savepoint_rollback()

    def savepoint_begin(self, name=None):
        if name is None:
            name = 's' + uuid4().hex
        self._savepoints.append(name)
        self.execute("SAVEPOINT %s", name)

    def savepoint_commit(self, name=None):
        if name is None:
            name = self._savepoints.pop()
        else:
            del self._savepoints[self._savepoints.index(name):]
        self.execute("RELEASE SAVEPOINT %s", name)

    def savepoint_rollback(self, name=None):
        if name is None:
            name = self._savepoints.pop()
        else:
            del self._savepoints[self._savepoints.index(name):]
        self.execute("ROLLBACK TO SAVEPOINT %s", name)

    def describe_table(self, table_name):
        return {}

    def qn(self, name):
        return self.compile(smartsql.Name(name))[0]

    def close(self):
        if self.connection:
            self.connection.close()
            self.connection = None

    @classmethod
    def register(cls, engine):
        def _deco(engine_cls):
            cls._engines[engine] = engine_cls
            return cls
        return _deco

    @classmethod
    def factory(cls, **kwargs):
        try:
            Cls = cls._engines[kwargs['engine']]
        except KeyError:
            Cls = resolve(kwargs['engine'])
        database = Cls(**kwargs)
        if 'django_alias' in kwargs:
            database.connection_factory = django_connection_factory
        return database


def django_connection_factory(django_alias, **kwargs):
    # self.django_conn.ensure_connection()
    from django.db import connections
    django_connection = connections[django_alias]
    if not django_connection.connection:
        django_connection.cursor()
    return django_connection.connection


@Database.register('sqlite3')
class SqliteDatabase(Database):

    placeholder = '?'
    compile = sqlite.compile

    def connection_factory(self, **kwargs):
        import sqlite3
        return sqlite3.connect(**kwargs)


@Database.register('mysql')
class MySQLDatabase(Database):

    compile = mysql.compile

    def connection_factory(self, **kwargs):
        import MySQLdb
        return MySQLdb.connect(**kwargs)

    def get_pk(self, table_name):
        cursor = self.execute("""
            SELECT COLUMN_NAME
            FROM   information_schema.KEY_COLUMN_USAGE
            WHERE  TABLE_SCHEMA = SCHEMA()
            AND    CONSTRAINT_NAME = 'PRIMARY'
            AND    TABLE_NAME = %s
            ORDER BY ORDINAL_POSITION;
        """, [table_name])
        return tuple(i[0] for i in cursor.fetchall())

    def describe_table(self, table_name):
        cursor = self.execute("""
            SELECT COLUMN_NAME, ORDINAL_POSITION, DATA_TYPE, IS_NULLABLE, COLUMN_DEFAULT, CHARACTER_MAXIMUM_LENGTH
            FROM   information_schema.COLUMNS
            WHERE  TABLE_SCHEMA = SCHEMA()
            AND    TABLE_NAME = %s
            ORDER BY ORDINAL_POSITION;
        """, [table_name])
        schema = collections.OrderedDict()
        for row in cursor.fetchall():
            col = {
                'column': row[0],
                'position': row[1],
                'data_type': row[2],
                'null': row[3].upper() == 'YES',
                # 'default': row[4],
                'max_length': row[5],
            }
            schema[col['column']] = col
        return schema

    def set_autocommit(self, autocommit=True):
        self.execute("SET autocommit={}".format(int(autocommit)))
        super(MySQLDatabase, self).set_autocommit(autocommit)


@Database.register('postgresql')
class PostgreSQLDatabase(Database):

    def connection_factory(self, **kwargs):
        import psycopg2
        return psycopg2.connect(**kwargs)

    def last_insert_id(self, cursor):
        cursor.execute("SELECT lastval()")
        return cursor.fetchone()[0]

    def get_pk(self, table_name):
        # https://wiki.postgresql.org/wiki/Retrieve_primary_key_columns
        cursor = self.execute("""
        SELECT a.attname, format_type(a.atttypid, a.atttypmod) AS data_type
        FROM   pg_index i JOIN pg_attribute a ON a.attrelid = i.indrelid
               AND a.attnum = ANY(i.indkey)
        WHERE  i.indrelid = %s::regclass
               AND i.indisprimary
        ORDER BY a.attnum;
        """, [table_name])
        return tuple(i[0] for i in cursor.fetchall())

    def describe_table(self, table_name):
        cursor = self.execute("""
            SELECT column_name, ordinal_position, data_type, is_nullable, column_default, character_maximum_length
            FROM   information_schema.columns
            WHERE  table_name = %s
            ORDER BY ordinal_position;
        """, [table_name])
        schema = collections.OrderedDict()
        for row in cursor.fetchall():
            col = {
                'column': row[0],
                'position': row[1],
                'data_type': row[2],
                'null': row[3].upper() == 'YES',
                # 'default': row[4],
                'max_length': row[5],
            }
            schema[col['column']] = col
        return schema

    def set_autocommit(self, autocommit=True):
        self.execute("SET AUTOCOMMIT = {}".format('ON' if autocommit else 'OFF'))
        super(PostgreSQLDatabase, self).set_autocommit(autocommit)


class Databases(object):

    def __init__(self, conf):
        self._settings = conf
        self._databases = local()

    def create_database(self, alias):
        return Database.factory(alias=alias, **self._settings[alias])

    def close(self):
        for alias in self:
            del self[alias]

    def __getitem__(self, alias):
        try:
            return getattr(self._databases, alias)
        except AttributeError:
            setattr(self._databases, alias, self.create_database(alias))
            return getattr(self._databases, alias)

    def __delitem__(self, alias):
        if hasattr(self._databases, alias):
            getattr(self._databases, alias).close()
            delattr(self._databases, alias)

    def __iter__(self):
        return iter(self._settings)


def get_db(alias=None):
    smartsql.warn('get_db', 'databases')
    return databases[alias or 'default']


databases = Databases(settings.DATABASES)



#!/usr/bin/env python
# Copyright (c) 2012-2013 Ivan Zakrevsky, Jared Kuolt and contributors
import os.path
from setuptools import setup, find_packages

setup(
    name='ascetic',
    version='0.7.2.17',

    packages = find_packages(),
    include_package_data=True,

    author="Ivan Zakrevsky",
    author_email="ivzak@yandex.ru",
    description="Lightweight python ORM (Object-relational mapper).",
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    url="https://bitbucket.org/emacsway/ascetic",
    license="MIT License",
    keywords = "ORM Database SQL",
    install_requires=[
        'sqlbuilder >= 0.7.9.8'
    ],
    classifiers = [
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



from __future__ import absolute_import
from weakref import WeakKeyDictionary, WeakValueDictionary

__all__ = ('Signal', 'pre_save', 'post_save', 'pre_delete', 'post_delete',
           'pre_init', 'post_init', 'class_prepared', 'field_conversion')


class Signal(object):
    def __init__(self):
        self._flush()

    @staticmethod
    def _make_id(target):
        if hasattr(target, '__func__'):
            return (id(target.__self__), id(target.__func__))
        return id(target)

    def connect(self, receiver, sender=None, weak=True, receiver_id=None):
        if not weak:
            self._weak_cache.add(receiver)
        if receiver_id is None:
            receiver_id = self._make_id(receiver)
        if sender not in self._receivers:
            self._receivers[sender] = WeakValueDictionary()
        self._receivers[sender][receiver_id] = receiver

    def disconnect(self, receiver=None, sender=None, receiver_id=None):
        if receiver_id is None:
            receiver_id = self._make_id(receiver)
        if receiver_id:
            try:
                self._weak_cache.discard(receiver or self._receivers[sender][receiver_id])
                del self._receivers[sender][receiver_id]
            except KeyError:
                pass
        else:
            raise ValueError('a receiver or a receiver_id must be provided')

    def send(self, sender, *args, **kwargs):
        responses = []
        if sender in self._receivers:
            receivers = list(self._receivers[sender].values())
            if sender is not None:
                receivers += list(self._receivers[None].values())
            for receiver in receivers:
                responses.append((receiver, receiver(sender, *args, **kwargs)))
        return responses

    def _flush(self):
        self._receivers = WeakKeyDictionary()
        self._weak_cache = set()


def connect(signal, sender=None, weak=True, receiver_id=None):
    def decorator(fn):
        signal.connect(fn, sender, weak, receiver_id)
        return fn
    return decorator

pre_save = Signal()
post_save = Signal()
pre_delete = Signal()
post_delete = Signal()
pre_init = Signal()
post_init = Signal()
class_prepared = Signal()
field_conversion = Signal()
